# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Tully Foote, Ken Conley

from __future__ import print_function
import subprocess
import sys

from rospkg.os_detect import OS_DEBIAN, OS_LINARO, OS_UBUNTU, OS_ELEMENTARY, OS_MX, OsDetect, read_os_release

from .pip import PIP_INSTALLER
from .gem import GEM_INSTALLER
from .source import SOURCE_INSTALLER
from ..installers import PackageManagerInstaller
from ..shell_utils import read_stdout

# apt package manager key
APT_INSTALLER = 'apt'


def register_installers(context):
    context.set_installer(APT_INSTALLER, AptInstaller())


def register_platforms(context):
    register_debian(context)
    register_ubuntu(context)

    # Aliases
    register_elementary(context)
    register_linaro(context)
    register_mx(context)


def register_debian(context):
    context.add_os_installer_key(OS_DEBIAN, APT_INSTALLER)
    context.add_os_installer_key(OS_DEBIAN, PIP_INSTALLER)
    context.add_os_installer_key(OS_DEBIAN, GEM_INSTALLER)
    context.add_os_installer_key(OS_DEBIAN, SOURCE_INSTALLER)
    context.set_default_os_installer_key(OS_DEBIAN, lambda self: APT_INSTALLER)
    context.set_os_version_type(OS_DEBIAN, OsDetect.get_codename)


def register_linaro(context):
    # Linaro is an alias for Ubuntu. If linaro is detected and it's not set as
    # an override force ubuntu.
    (os_name, os_version) = context.get_os_name_and_version()
    if os_name == OS_LINARO and not context.os_override:
        print('rosdep detected OS: [%s] aliasing it to: [%s]' %
              (OS_LINARO, OS_UBUNTU), file=sys.stderr)
        context.set_os_override(OS_UBUNTU, context.os_detect.get_codename())


def register_elementary(context):
    # Elementary is an alias for Ubuntu. If elementary is detected and it's
    # not set as an override force ubuntu.
    (os_name, os_version) = context.get_os_name_and_version()
    if os_name == OS_ELEMENTARY and not context.os_override:
        print('rosdep detected OS: [%s] aliasing it to: [%s]' %
              (OS_ELEMENTARY, OS_UBUNTU), file=sys.stderr)
        context.set_os_override(OS_UBUNTU, context.os_detect.get_codename())


def register_mx(context):
    # MX is an alias for Debian. If MX is detected and it's
    # not set as an override, force Debian.
    (os_name, os_version) = context.get_os_name_and_version()
    if os_name == OS_MX and not context.os_override:
        print('rosdep detected OS: [%s] aliasing it to: [%s]' %
              (OS_MX, OS_DEBIAN), file=sys.stderr)
        release_info = read_os_release()
        version = read_os_release()["VERSION"]
        context.set_os_override(OS_DEBIAN, version[version.find("(") + 1:version.find(")")])


def register_ubuntu(context):
    context.add_os_installer_key(OS_UBUNTU, APT_INSTALLER)
    context.add_os_installer_key(OS_UBUNTU, PIP_INSTALLER)
    context.add_os_installer_key(OS_UBUNTU, GEM_INSTALLER)
    context.add_os_installer_key(OS_UBUNTU, SOURCE_INSTALLER)
    context.set_default_os_installer_key(OS_UBUNTU, lambda self: APT_INSTALLER)
    context.set_os_version_type(OS_UBUNTU, OsDetect.get_codename)


def _read_apt_cache_showpkg(packages, exec_fn=None):
    """
    Output whether these packages are virtual package list providing package.
    If one package was not found, it gets returned as non-virtual.
    :param exec_fn: see `dpkg_detect`; make sure that exec_fn supports a
    second, boolean, parameter.
    """

    cmd = ['apt-cache', 'showpkg'] + packages
    if exec_fn is None:
        exec_fn = read_stdout

    std_out = exec_fn(cmd).splitlines()

    starts = []
    notfound = set()
    for p in packages:
        last_start = starts[-1] if len(starts) > 0 else 0
        try:
            starts.append(std_out.index('Package: %s' % p, last_start))
        except ValueError:
            notfound.add(p)
    starts.append(None)

    for p in packages:
        if p in notfound:
            yield p, False, None
            continue
        start = starts.pop(0)
        lines = iter(std_out[start:starts[0]])

        header = 'Package: %s' % p
        # proceed to Package header
        while next(lines) != header:
            pass

        # proceed to versions section
        while next(lines) != 'Versions: ':
            pass

        # virtual packages don't have versions
        if next(lines) != '':
            yield p, False, None
            continue

        # proceed to reserve provides section
        while next(lines) != 'Reverse Provides: ':
            pass

        pr = [line.split(' ', 2)[0] for line in lines]
        if pr:
            yield p, True, pr
        else:
            yield p, False, None


def dpkg_detect(pkgs, exec_fn=None):
    """
    Given a list of package, return the list of installed packages.

    :param pkgs: list of package names, optionally followed by a fixed version (`foo=3.0`)
    :param exec_fn: function to execute Popen and read stdout (for testing)
    :return: list elements in *pkgs* that were found installed on the system
    """
    ret_list = []
    # this is mainly a hack to support version locking for eigen.
    # we strip version-locking syntax, e.g. libeigen3-dev=3.0.1-*.
    # our query does not do the validation on the version itself.
    # This is a map `package name -> package name optionally with version`.
    version_lock_map = {}
    for p in pkgs:
        if '=' in p:
            version_lock_map[p.split('=')[0]] = p
        else:
            version_lock_map[p] = p
    cmd = ['dpkg-query', '-W', '-f=\'${Package} ${Status}\n\'']
    cmd.extend(version_lock_map.keys())

    if exec_fn is None:
        exec_fn = read_stdout
    std_out, std_err = exec_fn(cmd, True)
    std_out = std_out.replace('\'', '')
    pkg_list = std_out.split('\n')
    for pkg in pkg_list:
        pkg_row = pkg.split()
        if len(pkg_row) == 4 and (pkg_row[3] == 'installed'):
            ret_list.append(pkg_row[0])
    installed_packages = [version_lock_map[r] for r in ret_list]

    # now for the remaining packages check, whether they are installed as
    # virtual packages
    remaining = _read_apt_cache_showpkg(list(p for p in pkgs if p not in installed_packages))
    virtual = [n for (n, v, pr) in remaining if v and len(dpkg_detect(pr)) > 0]

    return installed_packages + virtual


def _iterate_packages(packages, reinstall):
    for entry in _read_apt_cache_showpkg(packages):
        p, is_virtual, providers = entry
        if is_virtual:
            installed = []
            if reinstall:
                installed = dpkg_detect(providers)
                if len(installed) > 0:
                    for i in installed:
                        yield i
                    continue  # don't ouput providers
            yield providers
        else:
            yield p


class AptInstaller(PackageManagerInstaller):

    """
    An implementation of the Installer for use on debian style
    systems.
    """

    def __init__(self):
        super(AptInstaller, self).__init__(dpkg_detect)

    def get_version_strings(self):
        output = subprocess.check_output(['apt-get', '--version'])
        version = output.splitlines()[0].split(' ')[1]
        return ['apt-get {}'.format(version)]

    def _get_install_commands_for_package(self, base_cmd, package_or_list):
        def pkg_command(p):
            return self.elevate_priv(base_cmd + [p])

        if isinstance(package_or_list, list):
            return [pkg_command(p) for p in package_or_list]
        else:
            return pkg_command(package_or_list)

    def get_install_command(self, resolved, interactive=True, reinstall=False, quiet=False):
        packages = self.get_packages_to_install(resolved, reinstall=reinstall)
        if not packages:
            return []
        base_cmd = ['apt-get', 'install']
        if not interactive:
            base_cmd.append('-y')
        if quiet:
            base_cmd.append('-qq')

        return [self._get_install_commands_for_package(base_cmd, p) for p in _iterate_packages(packages, reinstall)]



# Copyright (c) 2019, LG Electronics, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Andre Rosa/andre.rosa@lge.com

import subprocess
from rospkg.os_detect import OS_OPENEMBEDDED, OsDetect
from ..installers import PackageManagerInstaller
OPKG_INSTALLER = 'opkg'


def register_installers(context):
    context.set_installer(OPKG_INSTALLER, OpkgInstaller())


def register_platforms(context):
    register_oe(context)


def register_oe(context):
    context.add_os_installer_key(OS_OPENEMBEDDED, OPKG_INSTALLER)
    context.set_default_os_installer_key(OS_OPENEMBEDDED, lambda self: OPKG_INSTALLER)
    context.set_os_version_type(OS_OPENEMBEDDED, OsDetect.get_codename)


def opkg_detect(pkgs, exec_fn=None):
    """
    Given a list of package, return the list of installed packages.
    NOTE: These are stubs currently and will be filled after semantics are fully defined.

    :param pkgs: list of package names, optionally followed by a fixed version (`foo=3.0`)
    :param exec_fn: function to execute Popen and read stdout (for testing)
    :return: list elements in *pkgs* that were found installed on the system
    """
    raise NotImplementedError("opkg_detect is not implemented yet")


class OpkgInstaller(PackageManagerInstaller):
    """
    An implementation of the Installer for use on oe systems.
    NOTE: These are stubs currently and will be filled after semantics are fully defined.
    """

    def __init__(self):
        super(OpkgInstaller, self).__init__(opkg_detect)

    def get_version_strings(self):
        output = subprocess.check_output(['opkg', '--version'])
        version = output.splitlines()[0].split(' ')[2]
        return [('opkg {}').format(version)]

    def get_install_command(self, resolved, interactive=True, reinstall=False, quiet=False):
        raise NotImplementedError('get_install_command is not implemented yet')



# Copyright (c) 2018, SEQSENSE, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Atsushi Watanabe/atsushi.w@ieee.org

import os
import traceback
from mock import Mock, patch

import rospkg.os_detect


def get_test_dir():
    # not used yet
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'alpine'))


def test_apk_detect():
    from rosdep2.platforms.alpine import apk_detect

    m = Mock(return_value='')
    expected = []
    val = apk_detect([], exec_fn=m)
    assert val == expected, 'Result was: %s' % val
    m.assert_not_called()

    m = Mock(return_value='')
    expected = []
    val = apk_detect(['a'], exec_fn=m)
    assert val == expected, 'Result was: %s' % val
    m.assert_called_with(['apk', 'info', '--installed', 'a'])

    m = Mock(return_value='\n'.join(['a', 'b']))
    expected = ['a', 'b']
    val = apk_detect(['a', 'b'], exec_fn=m)
    assert val == expected, 'Result was: %s' % val
    m.assert_called_with(['apk', 'info', '--installed', 'a', 'b'])


def test_ApkInstaller():
    from rosdep2.platforms.alpine import ApkInstaller

    @patch.object(ApkInstaller, 'get_packages_to_install')
    def test(mock_method):
        installer = ApkInstaller()
        mock_method.return_value = []
        assert [] == installer.get_install_command(['nonexistingfakepackage'])

        mock_method.return_value = ['a-dev', 'b-dev']

        expected = [['sudo', '-H', 'apk', 'add', 'a-dev', 'b-dev']]
        val = installer.get_install_command(['notused'], interactive=False, quiet=False)
        assert val == expected, 'Result was: %s' % val

        expected = [['sudo', '-H', 'apk', 'add', '--interactive', 'a-dev', 'b-dev']]
        val = installer.get_install_command(['notused'], interactive=True, quiet=False)
        assert val == expected, 'Result was: %s' % val

        expected = [['sudo', '-H', 'apk', 'add', '--quiet', 'a-dev', 'b-dev']]
        val = installer.get_install_command(['notused'], interactive=False, quiet=True)
        assert val == expected, 'Result was: %s' % val

    try:
        test()
    except AssertionError:
        traceback.print_exc()
        raise



# -*- coding: utf-8 -*-

# Copyright (c) 2011, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Ken Conley/kwc@willowgarage.com

import os
import traceback

from mock import call
from mock import Mock
from mock import patch


def get_test_dir():
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'osx'))


def is_port_installed_tripwire():
    # don't know the correct answer, but make sure this does not throw
    from rosdep2.platforms.osx import is_port_installed
    assert is_port_installed() in [True, False]


def is_brew_installed_tripwire():
    # don't know the correct answer, but make sure this does not throw
    from rosdep2.platforms.osx import is_brew_installed
    assert is_brew_installed() in [True, False]


def make_resolutions(package_list):
    from rosdep2.platforms.osx import HomebrewResolution
    return list(map(lambda pkg: HomebrewResolution(pkg, [], []), package_list))


def make_resolutions_options(package_list):
    from rosdep2.platforms.osx import HomebrewResolution
    return list(map(lambda pkg: HomebrewResolution(pkg[0], pkg[1], pkg[2]), package_list))


def brew_command(command):
    if command[1] == 'list':
        with open(os.path.join(get_test_dir(), 'brew-list-output'), 'r') as f:
            return f.read()
    elif command[1] == 'info':
        pkg = command[2]
        with open(os.path.join(get_test_dir(), 'brew-info-output'), 'r') as f:
            output = f.readlines()
        for line in output:
            res = line.split(':', 1)
            if res[0] == pkg:
                return res[1]
    return ''


def test_brew_detect():
    from rosdep2.platforms.osx import brew_detect

    m = Mock()
    m.return_value = ''
    val = brew_detect([], exec_fn=m)
    assert val == [], val

    m = Mock()
    m.return_value = ''
    val = brew_detect(make_resolutions(['tinyxml']), exec_fn=m)
    assert val == [], val
    # make sure our test harness is based on the same implementation
    m.assert_called_with(['brew', 'list'])
    assert m.call_args_list == [call(['brew', 'list'])], m.call_args_list

    m = Mock()
    m.side_effect = brew_command
    val = brew_detect(make_resolutions(['apt', 'subversion', 'python', 'bazaar']), exec_fn=m)
    # make sure it preserves order
    expected = make_resolutions(['subversion', 'bazaar'])
    assert set(val) == set(expected), val
    assert val == expected, val
    assert len(val) == len(set(val)), val


def test_HomebrewInstaller():
    from rosdep2.platforms.osx import HomebrewInstaller

    @patch('rosdep2.platforms.osx.is_brew_installed')
    @patch.object(HomebrewInstaller, 'remove_duplicate_dependencies')
    @patch.object(HomebrewInstaller, 'get_packages_to_install')
    def test(mock_get_packages_to_install, mock_remove_duplicate_dependencies, mock_brew_installed):
        mock_brew_installed.return_value = True

        installer = HomebrewInstaller()
        mock_get_packages_to_install.return_value = []
        mock_remove_duplicate_dependencies.return_value = mock_get_packages_to_install.return_value
        assert [] == installer.get_install_command(make_resolutions(['fake']))

        mock_get_packages_to_install.return_value = make_resolutions(['subversion', 'bazaar'])
        mock_remove_duplicate_dependencies.return_value = mock_get_packages_to_install.return_value
        expected = [['brew', 'install', 'subversion'],
                    ['brew', 'install', 'bazaar']]
        # brew is always non-interactive
        for interactive in [True, False]:
            val = installer.get_install_command(['whatever'], interactive=interactive)
            assert val == expected, val

        expected = [['brew', 'uninstall', '--force', 'subversion'],
                    ['brew', 'install', 'subversion'],
                    ['brew', 'uninstall', '--force', 'bazaar'],
                    ['brew', 'install', 'bazaar']]
        val = installer.get_install_command(['whatever'], reinstall=True)
        assert val == expected, val

        mock_get_packages_to_install.return_value = make_resolutions_options(
            [('subversion', ['foo', 'bar'], ['baz']), ('bazaar', [], ['--with-quux'])])
        mock_remove_duplicate_dependencies.return_value = mock_get_packages_to_install.return_value
        expected = [['brew', 'install', 'subversion', 'foo', 'bar', 'baz'],
                    ['brew', 'install', 'bazaar', '--with-quux']]
        val = installer.get_install_command(['whatever'])
        assert val == expected, val

        try:
            mock_get_packages_to_install.return_value = eval("make_resolutions_options([('subversion', [u'f´´ßß', u'öäö'], []), (u'bazaar', [], [u'tüü'])])")
        except SyntaxError:
            # Python 3.2, u'...' is not allowed, but string literals are unicode
            mock_get_packages_to_install.return_value = make_resolutions_options(
                [('subversion', ['f´´ßß', 'öäö'], []), ('bazaar', [], ['tüü'])])
        mock_remove_duplicate_dependencies.return_value = mock_get_packages_to_install.return_value
        try:
            expected = eval("[['brew', 'install', 'subversion', u'f´´ßß', u'öäö'], ['brew', 'install', 'bazaar', u'tüü']]")
        except SyntaxError:
            # Python 3.2, u'...' is not allowed, but string literals are unicode
            expected = [['brew', 'install', 'subversion', 'f´´ßß', 'öäö'],
                        ['brew', 'install', 'bazaar', 'tüü']]
        val = installer.get_install_command(['whatever'])
        assert val == expected, val
    try:
        test()
    except AssertionError:
        traceback.print_exc()
        raise



# Copyright (c) 2010, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Original from cygwin.py by Tingfan Wu tingfan@gmail.com
# Modified for FreeBSD by Rene Ladan rene@freebsd.org
# Updated for FreeBSD with pkg by Trenton Schulz trentonw@ifi.uio.no

from rospkg.os_detect import OS_FREEBSD

from .pip import PIP_INSTALLER
from ..installers import PackageManagerInstaller
from ..shell_utils import read_stdout

PKG_INSTALLER = 'pkg'


def register_installers(context):
    context.set_installer(PKG_INSTALLER, PkgInstaller())


def register_platforms(context):
    context.add_os_installer_key(OS_FREEBSD, PKG_INSTALLER)
    context.add_os_installer_key(OS_FREEBSD, PIP_INSTALLER)
    context.set_default_os_installer_key(OS_FREEBSD, lambda self: PKG_INSTALLER)


def pkg_detect_single(p, exec_fn):
    if p == "builtin":
        return True

    cmd = ['/usr/sbin/pkg', 'query', '%n', p]
    std_out = exec_fn(cmd)
    return std_out.split() != []


def pkg_detect(packages, exec_fn=None):
    if exec_fn is None:
        exec_fn = read_stdout
    return [p for p in packages if pkg_detect_single(p, exec_fn)]


class PkgInstaller(PackageManagerInstaller):
    """
    An implementation of the Installer for use on FreeBSD-style
    systems.
    """

    def __init__(self):
        super(PkgInstaller, self).__init__(pkg_detect)

    def get_install_command(self, resolved, interactive=True, reinstall=False, quiet=False):
        packages = self.get_packages_to_install(resolved, reinstall=reinstall)
        if not packages:
            return []
        else:
            return [self.elevate_priv(['/usr/sbin/pkg', 'install', '-y']) + packages]



# Copyright 2015 Open Source Robotics Foundation, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from .constants import RESOURCE_INDEX_SUBFOLDER

from .search_paths import get_search_paths


def get_resources(resource_type):
    """
    Get the resource names of all resources of the specified type.

    :param resource_type: the type of the resource
    :type resource_type: str
    :returns: dict of resource names to the prefix path they are in
    :raises: :exc:`EnvironmentError`
    """
    assert resource_type, 'The resource type must not be empty'
    resources = {}
    for path in get_search_paths():
        resource_path = os.path.join(path, RESOURCE_INDEX_SUBFOLDER, resource_type)
        if os.path.isdir(resource_path):
            for resource in os.listdir(resource_path):
                # Ignore subdirectories, and anything starting with a dot
                if os.path.isdir(os.path.join(resource_path, resource)) \
                        or resource.startswith('.'):
                    continue
                if resource not in resources:
                    resources[resource] = path
    return resources



# Copyright (c) 2009, Willow Garage, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Willow Garage, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author Nikolay Nikolov/niko.b.nikolov@gmail.com

import subprocess
import os

from ..core import InstallFailed
from .pip import PIP_INSTALLER
from ..installers import PackageManagerInstaller
from .source import SOURCE_INSTALLER
from ..shell_utils import read_stdout

SLACKWARE_OS_NAME = 'slackware'
SBOTOOLS_INSTALLER = 'sbotools'
SLACKPKG_INSTALLER = 'slackpkg'


def register_installers(context):
    context.set_installer(SBOTOOLS_INSTALLER, SbotoolsInstaller())
    context.set_installer(SLACKPKG_INSTALLER, SlackpkgInstaller())


def register_platforms(context):
    context.add_os_installer_key(SLACKWARE_OS_NAME, SBOTOOLS_INSTALLER)
    context.add_os_installer_key(SLACKWARE_OS_NAME, PIP_INSTALLER)
    context.add_os_installer_key(SLACKWARE_OS_NAME, SOURCE_INSTALLER)
    context.add_os_installer_key(SLACKWARE_OS_NAME, SLACKPKG_INSTALLER)
    context.set_default_os_installer_key(SLACKWARE_OS_NAME, lambda self: SBOTOOLS_INSTALLER)


def sbotools_available():
    if not os.path.exists('/usr/sbin/sboinstall'):
        return False
    return True


def sbotools_detect_single(p):
    pkg_list = read_stdout(['ls', '/var/log/packages'])
    p = subprocess.Popen(['grep', '-i', '^' + p], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.communicate(pkg_list)
    return not p.returncode


def sbotools_detect(packages):
    return [p for p in packages if sbotools_detect_single(p)]


class SbotoolsInstaller(PackageManagerInstaller):

    def __init__(self):
        super(SbotoolsInstaller, self).__init__(sbotools_detect)

    def get_install_command(self, resolved, interactive=True, reinstall=False, quiet=False):
        if not sbotools_available():
            raise InstallFailed((SBOTOOLS_INSTALLER, 'sbotools is not installed'))

        packages = self.get_packages_to_install(resolved, reinstall=reinstall)
        if not packages:
            return []

        cmd = ['sboinstall']

        return [self.elevate_priv(cmd + [p] + ['-j']) for p in packages]


def slackpkg_available():
    if not os.path.exists('/usr/sbin/slackpkg'):
        return False
    return True


def slackpkg_detect_single(p):
    return not subprocess.call(['slackpkg', 'search', p], stdout=subprocess.PIPE, stderr=subprocess.PIPE)


def slackpkg_detect(packages):
    return [p for p in packages if slackpkg_detect_single(p)]


class SlackpkgInstaller(PackageManagerInstaller):

    def __init__(self):
        super(SlackpkgInstaller, self).__init__(slackpkg_detect)

    def get_install_command(self, resolved, interactive=True, reinstall=False, quiet=False):
        # slackpkg does not provide non-interactive mode
        packages = self.get_packages_to_install(resolved, reinstall=reinstall)
        if not packages:
            return []
        else:
            return [self.elevate_priv(['slackpkg', 'install', p]) for p in packages]



import os
from setuptools import setup

exec(open(os.path.join(os.path.dirname(__file__), 'src', 'rosdep2', '_version.py')).read())

setup(
    name='rosdep',
    version=__version__,  # noqa:F821
    packages=['rosdep2', 'rosdep2.ament_packages', 'rosdep2.platforms'],
    package_dir={'': 'src'},
    install_requires=['catkin_pkg >= 0.4.0', 'rospkg >= 1.1.10', 'rosdistro >= 0.7.5', 'PyYAML >= 3.1'],
    test_suite='nose.collector',
    test_requires=['mock', 'nose >= 1.0'],
    scripts=['scripts/rosdep', 'scripts/rosdep-source'],
    author='Tully Foote, Ken Conley',
    author_email='tfoote@osrfoundation.org',
    url='http://wiki.ros.org/rosdep',
    keywords=['ROS'],
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License'],
    description='rosdep package manager abstraction tool for ROS',
    long_description='Command-line tool for installing system '
                     'dependencies on a variety of platforms.',
    license='BSD'
)
